@@ -688,30 +688,6 @@ class Sf:
         
         return result_dict
     
-    ## TODO: This function can be made much more efficient by doing all of the object, permission, and profile metadata retrievals in one go
-    async def field_permission_user_bulk_check(self, objects: list, permissions: list,  users= [], ):
-        data = {}
-        task_dict = {}
-        for object in objects:
-            for permission in permissions:
-                task_dict[f"{object};;{permission}"] = asyncio.create_task(self.field_permission_user_check(object, permission, users))
-        
-        gathered_tasks = await asyncio.gather(*task_dict.values())
-        
-        task_results_dict = dict(zip(task_dict.keys(), gathered_tasks))
-        
-        
-        result = []
-        
-        for key, value in task_results_dict.items():
-            object, permission = key.split(';;')
-            value['Object API Name'] = object
-            value['Permission'] = permission
-            result.append(value)
-
-        result_pd = pd.concat(result, ignore_index=True)
-        result_pd.columns = ['UserName','Profile','Field API Name','Permission Set','Object API Name','Permission']
-        return result_pd
 
     async def entity_access_bulk_query(self, entity_types: list):
         async_entity_access_query = async_wrap(self.entity_access_query)
@@ -878,12 +854,15 @@ class Sf:
         permissionable_fields = {}
         nonpermissionable_fields = {}
         async_permissionable_fields =  async_wrap(self.object_describe)
+        async_metadata_read = async_wrap(self.metadata_read)
+        
         
         task_dict = {}
         task_result_dict = {}
         
         for object_name in object_list:
             task_dict[object_name] = async_permissionable_fields(object_name)
+            task_dict[object_name + '_metadata'] = async_metadata_read('CustomObject', object_name)
             
         task_results = await asyncio.gather(*task_dict.values())
         
@@ -892,7 +871,8 @@ class Sf:
         
         
         for object in object_list:
-            permissionable_fields[object] = task_result_dict[object]['fields'].T[task_result_dict[object]['fields'].T['permissionable'] == True].index.tolist()
+            permissionable_fields[object] = pd.DataFrame(task_result_dict[object + "_metadata"]['fields'][0])['fullName'].to_list()
+            
             nonpermissionable_fields[object] = task_result_dict[object]['fields'].T[task_result_dict[object]['fields'].T['permissionable'] == False].index.tolist()
         return permissionable_fields, nonpermissionable_fields, task_result_dict
     
@@ -1143,17 +1123,20 @@ class Sf:
         
         return {'permissionSetPermissions': permissionSetPermissions, 'profilePermissions': profilePermissions}
 
-    async def field_permission_user_check(self, object, permission, users=[]):
+    async def field_permission_user_check(self, objects, permissions, users=[]):
         # Similar to the field_permission_check function, but this one will return a list of users that have the specified permission on the specified object
         
+        object_query = "','".join(objects)
+        permission_query = ' OR '.join([f"({permission} = true)" for permission in permissions])
+
         relevant_ps_and_profiles = transform_sf_result_set_rec(self.sf.query_all(f"""
                                                 SELECT ParentId,
                                                         Field,
                                                         PermissionsEdit,
                                                         PermissionsRead
                                                 FROM FieldPermissions
-                                                WHERE SObjectType IN ('{object}') AND
-                                                ({permission} = true)
+                                                WHERE SObjectType IN ('{object_query}') AND
+                                                ({permission_query})
                                                 """)['records'])
 
         relevant_ps_and_profiles['Object API Name'] = relevant_ps_and_profiles['FieldPermissions.Field'].apply(lambda row: row.split('.')[0])
@@ -1192,7 +1175,6 @@ class Sf:
                 profile_metadata_dict["System Administrator" if profile == 'Admin' else profile] = pd.DataFrame(task_result_list[profiles.index(profile)]['fieldPermissions'][0])
     
         data = data.merge(relevant_ps_and_profiles, left_on='PermissionSet.Id', right_on='FieldPermissions.ParentId', how='left').copy()
-
         if (len(data) > 0):
                 
                 profile_violations = data[data['PermissionSet.IsOwnedByProfile']== True][['User.Username', 'Profile.Name']].drop_duplicates().copy()
@@ -1201,23 +1183,38 @@ class Sf:
                 profile_metadata_pd.rename(columns={'Profile': 'Profile.Name'}, inplace=True)
                 profile_metadata_pd['Field API Name'] = profile_metadata_pd['field'].str.split('.').str[1]
                 profile_metadata_pd['Object API Name'] = profile_metadata_pd['field'].str.split('.').str[0]
-                profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['Object API Name'] == object].copy()
-                profile_metadata_pd.drop(columns=['field', 'Object API Name'], inplace=True)
+                profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['Object API Name'].isin(objects)].copy()
+                profile_metadata_pd.drop(columns=['field'], inplace=True)
 
-
-                if permission == 'PermissionsEdit':
-                        profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['editable'] == True].copy()
-                        profile_metadata_pd.drop(columns=['editable', 'readable'], inplace=True)
-                elif permission == 'PermissionsRead':
-                        profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['readable'] == True].copy()
-                        profile_metadata_pd.drop(columns=['editable', 'readable'], inplace=True)
+                # if permission == 'PermissionsEdit':
+                #         profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['editable'] == True].copy()
+                #         profile_metadata_pd.drop(columns=['editable', 'readable'], inplace=True)
+                # elif permission == 'PermissionsRead':
+                #         profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['readable'] == True].copy()
+                #         profile_metadata_pd.drop(columns=['editable', 'readable'], inplace=True)
                 
                 profile_violations = profile_violations.merge(profile_metadata_pd, on='Profile.Name', how='left').copy()
-                permission_set_violations = data[data['PermissionSet.IsOwnedByProfile']== False][['User.Username', 'PermissionSet.Label', 'Field API Name']]
-        
-                return pd.merge(profile_violations, permission_set_violations, on=['User.Username','Field API Name'], how='outer')
+                profile_editable = profile_violations[profile_violations['editable'] == True].drop(columns=['editable','readable']).copy()
+                profile_editable['Permission'] = 'PermissionsEdit'
+                profile_readable = profile_violations[profile_violations['readable'] == True].drop(columns=['editable','readable']).copy()
+                profile_readable['Permission'] = 'PermissionsRead'
+
+                profile_fls_final = pd.concat([profile_editable, profile_readable])
+                profile_fls_final.columns = ['UserName','Profile','Field API Name','Object API Name','Permission']
+                
+                permission_set_violations = data[data['PermissionSet.IsOwnedByProfile']== False][['User.Username', 'PermissionSet.Label', 'Object API Name', 'FieldPermissions.PermissionsEdit','FieldPermissions.PermissionsRead','Field API Name']]
+                
+                ps_editable = permission_set_violations[permission_set_violations['FieldPermissions.PermissionsEdit'] == True].drop(columns=['FieldPermissions.PermissionsEdit','FieldPermissions.PermissionsRead']).copy()
+                ps_editable['Permission'] = 'PermissionsEdit'
+                ps_readable = permission_set_violations[permission_set_violations['FieldPermissions.PermissionsRead'] == True].drop(columns=['FieldPermissions.PermissionsEdit','FieldPermissions.PermissionsRead']).copy()
+                ps_readable['Permission'] = 'PermissionsRead'
+                
+                ps_fls_final = pd.concat([ps_editable, ps_readable])
+                ps_fls_final.columns = ['UserName','Permission Set','Object API Name','Field API Name', 'Permission']
+                
+                return pd.merge(profile_fls_final, ps_fls_final, on=['UserName','Object API Name', 'Field API Name','Permission'], how='outer')
         else:
-                return pd.DataFrame(columns=['User.Username', 'Profile.Name', 'Field API Name', 'PermissionSet.Label'])
+                return pd.DataFrame(columns=['UserName', 'Profile', 'Field API Name', 'Object API Name','Permission', 'Permission Set'])
             
             
     # Retrieves the Page Layout by Profile by Record Type data for all objects in the org
