@@ -1,6 +1,8 @@
 try:
     from salesforce.utilities.df_utils import *
     from salesforce.utilities.async_utils import *
+    from salesforce.utilities.sf_query_processors import *
 except:
     from utilities.df_utils import *
-    from utilities.async_utils import *
\ No newline at end of file
+    from utilities.async_utils import *
+    from utilities.sf_query_processors import *
\ No newline at end of file
